from random import randint
def печать (текст, стиль="", красный=False, зелёный=False,рандомный=False, жёлтый=False, синий=False, золотой=False, фиолетовый=False, не_печатать=False):
    def определить_стиль(стиль: str):
        стиль = стиль.lower()
        результат = ""
        if "ж" in стиль:
            результат += ";1"
        if "к" in стиль:
            результат += ";3"
        if "ч" in стиль:
            результат += ";4"
        if "з" in стиль:
            результат += ";9"
        return результат

    итоговый_стиль = определить_стиль(стиль)
    if красный:
        цвет = "255;0;0"
    elif зелёный:
        цвет = "0;255;0"
    elif синий:
        цвет = "0;0;255"
    elif рандомный:
        результат = ""
        for буква in текст:
            К = randint(0, 255)
            З = randint(0, 255)
            С = randint(0, 255)
            результат += f"\x1B[38;2;{К};{З};{С}{итоговый_стиль}m{буква}"
        результат += "\x1B[0m" 
#    результат = f"\x1B[38;2;{цвет};1m{текст}\x1B[0m"
        print(результат)
        return
    else:
        цвет = "255;255;255"
    результат = f"\x1B[38;2;{цвет}{итоговый_стиль}m{текст}\x1B[0m"
    if не_печатать:
        return результат
    print(результат)
    # \x1B[ - особый символ, который включает режим другого цвета
    # 38;2; - говорит о том, что следующие 3 команды будут в форме RGB
    # после цвета ;1 - особый стиль 
    # \x1B[0m - всё, что будет написано после этого символа будет стандартным цветом. Если не будет этой строчки, то все последующие print  будут печататься тем же цветом.

if __name__ == "__main__":
#    печать("Этот текст был вызван внутри файла", красный=True, стиль="жчз")
#    печать("Этот текст был вызван внутри файла", красный=True, стиль="ж")
#    печать("Этот текст был вызван внутри файла", красный=True, стиль="ч")
#    печать("Этот текст был вызван внутри файла", красный=True, стиль="з")
#    печать("Этот текст был вызван внутри файла", красный=True, стиль="к")
#    print("это обычный белый цвет")
    красный = печать("красный текст", красный=True, не_печатать=True)
    зелёный = печать("зелёный текст", зелёный=True, не_печатать=True)
    print(f"{красный} и {зелёный} текст в одной строке")



РАЗДЕЛИТЕЛЬ_ПАР = "$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"
РАЗДЕЛИТЕЛЬ_ЗНАЧЕНИЙ = "№№№№№№№№№№№№№№№№№№№№№№№№№№№№№№№"
def записать_в_файл(текст: str, путь: str ):
    try:
        with open(путь, mode="w", encoding = "utf-8") as файл:
            файл.write(текст)
    except Exception as ошибка:
        print(ошибка)
        return False
    return True
def считать_из_файла(путь : str):
    try:
        with open(путь, mode="r", encoding = "utf-8") as файл:
            return файл.read()
            файл.write(текст)
    except Exception as ошибка:
        print(ошибка)
        return False
def словарь_в_текст(словарь: dict):
    результат = ""
    for пара in словарь.items():
        ключ, значение = пара
        текстовая_пара = f"{ключ}{РАЗДЕЛИТЕЛЬ_ЗНАЧЕНИЙ}{значение}"
        результат += текстовая_пара + РАЗДЕЛИТЕЛЬ_ПАР
    return результат[:-len (РАЗДЕЛИТЕЛЬ_ПАР)]
def текст_в_словарь(текст: str):
    результат = {}
    пары = текст.split(РАЗДЕЛИТЕЛЬ_ПАР)
    for пара in пары:
        ключ, значение = пара.split(РАЗДЕЛИТЕЛЬ_ЗНАЧЕНИЙ, 1)
        результат[ключ] = значение
    return результат
def преобразовать_все_словари_в_текст(список_словарей: list):
    текст = ""
    for словарь in список_словарей:
        текст += словарь_в_текст(словарь) + "\n"
    return текст
def преобразовать_весь_текст_в_словари(текст: str):
    список = []
    for строка in текст.splitlines():
        словарь = текст_в_словарь(строка)
        список.append(словарь)
    return список
if __name__ == "__main__":
    список_словарей =[
        {"ключ1":"значение1", "ключ2":"значение2"},
        {"ключ3":"значение3", "ключ4":"значение4"}
    ]
    текст = преобразовать_все_словари_в_текст(список_словарей)
    обратно = преобразовать_весь_текст_в_словари(текст)
    print(список_словарей == обратно)
#    ответ = преобразовать_словарь_в_текст(подготовленный)
#    словарь = преобразовать_текст_в_словарь(ответ)
#    print(словарь == подготовленный)
#    print(словарь)
#    print(подготовленный)
#if __name__ == "__main__":
#    путь_к_файлу = r"W:\Users\GR1\Desktop\Ученики\антонова юлиана\модуль\файл из питона.txt"
#    записать_в_файл("питоновский текстик", путь_к_файлу)
#этот_файлик = __file__
#import os
#эта_папка = os.path.dirname(этот_файлик)
#файл_с_текстом = os.path.join(эта_папка, "здесь хранится текст.txt")
#print(считать_из_файла(файл_с_текстом))



from терминал import печать
def содержит(что, где, игнорировать_регистр=False):
    # функция проверяет, что хотя бы один элемент из "что" (или вся переменная целиком) содержится хотя бы в одном элементе из "где" (или является частью самой переменной "где")

    # функция возвращает True или False
    # переменные "что" и "где" могут быть двух видов:
        # 1. итерируемые объекты (кортеж или список), всеми элементами которых являются строки (текст)
        # 2. сама переменная может быть строкой (текстом)
        # при нарушении любого из этих пунктов будет вызвана ошибка с описанием проблемы
    # переменная "игнорировать_регистр" может принимать только True, или быть по умолчанию False. При передаче другого типа данных будет вызвана ошибка.
    # при получении в "что" или "где" пустых строк или пустых списков будет возвращаться False
    if type(что) not in [str, list, tuple]:
        ошибка = f'Переменная "что" может быть только текстом, списком или кортежем. По факту у неё тип {type(что)}'
        raise TypeError(печать(ошибка, красный=True, стиль="жч", не_печатать=True))
    if type(где) not in [str, list, tuple]:
        ошибка = f'Переменная "где" может быть только текстом, списком или кортежем. По факту у неё тип {type(где)}'
        raise TypeError(печать(ошибка, красный=True, стиль="жч", не_печатать=True))
    if type(игнорировать_регистр) != bool:
        ошибка = f'Переменная "игнорировать_регистр" может быть только True или False, списком или кортежем. По факту у неё тип {type(игнорировать_регистр)}'
        raise TypeError(печать(ошибка, красный=True, стиль="жч", не_печатать=True))   
    if type(что) != str:
        for элемент in что:
            if type(элемент) != str:
                ошибка = f'В переменной "что" может быть только True или False. По факту у неё тип {type(что)}'
                raise TypeError(печать(ошибка, красный=True, стиль="жч", не_печатать=True))  
    if type(где) != str:
        for элемент in где:
            if type(элемент) != str:
                ошибка = f'В переменной "где" может быть только True или False. По факту у неё тип {type(где)}'
                raise TypeError(печать(ошибка, красный=True, стиль="жч", не_печатать=True))  

    if игнорировать_регистр:
        if type(что) == str:
            что = что.lower()
        else:
            новый_список = []
            for элемент in что:
                новый_список.append(элемент.lower())
            что = новый_список
    if игнорировать_регистр:
        if type(где) == str:
            где = где.lower()
        else:
            новый_список = []
            for элемент in где:
                новый_список.append(элемент.lower())
            гд = новый_список
    if type(что) == type(где) == str:
        if что in где:
            return True
    if type(что) == str and type(где) != str:
        for элемент in где:
            if что in элемент:
                return True
    if type(что) != str and type(где) == str:
        for элемент in что:
            if элемент in где:
                return True
    if type(что) != str and type(где) != str:
        for элемент_что in что:
            for элемент_где in где:
                if элемент_что in элемент_где:
                    return True
            
    return False
    # TDD - Test Driven Development
    # разработка функций после написания тестов к ней
    # согласно этому принципу нужно сначала написать тесты, которые проверят каждый аспект работы функции. каждый раз при доработке этой функции необходимо заново запускать тесты - есть они все пройдены, значит функция точно работает правильно.
    # при разработке тестов недопустимо использование вычисляемых переменных - тестируемая функция принять заранее подготовленные данные, а тест проверяет, согласуется ли её ответ с ожиданием.
    # например, при тестировании функции "сумма" ей в аргументы передаются числа 15 и 10, и сравнивается её ответ с числом 25, а не с вычислением 15 + 10
def тесты():
    пройдено = []
    не_пройдено = []
    def тест(что, где, игнорировать_регистр, ожидаемый_ответ, название_теста):
        try:
            ответ = содержит(что, где, игнорировать_регистр)
            текст_ошибки = None
        except Exception as ошибка:
            текст_ошибки = ошибка
            ответ = "ошибка"
        if ответ == ожидаемый_ответ:
            печать(название_теста, зелёный = True)
            пройдено.append(1)
        else:

            печать(название_теста, красный = True)
            print(f"ожидаемый - {ожидаемый_ответ}")
            print(f"ответ     - {ответ}")
            if текст_ошибки:
                print(текст_ошибки)
            print()
            не_пройдено.append("-")
    
    тест(123, "где", True, "ошибка", "неправильный тип что")
    тест("что", 123, True, "ошибка", "неправильный тип где")
    тест("что", "где", "текст", "ошибка", "неправильный тип регистр - игнорировать регистр")
#    тест("что", "где", True, None, "все типы верны, функция ничего не вернула")
    тест(["текст", "не цифра", 23], "где", False, "ошибка", "в переменной 'что' есть не str")
    тест(
        что="что",
        где=["текст", "не цифра", 23],
        игнорировать_регистр=False,
        ожидаемый_ответ="ошибка",
        название_теста='в переменной "где" есть не str'
    )
    варианты = [
        ["сырники", "кто-то ест Сырники", True, True, "что - строка; где - строка; игн_регистр - True; ответ - True"],
        ["сырники", "кто-то ест грибы", True, False, "что - строка; где - строка; игн_регистр - True; ответ - False"],
        ["Сырники", "кто-то ест Сырники", False, True, "что - строка; где - строка; игн_регистр - False; ответ - True"],
        ["сырники", "кто-то ест Сырники", False, False, "что - строка; где - строка; игн_регистр - False; ответ - False"],
        ["Сырники", ["кто-то ест Сырники", "кто-то ест грибы"], False, True, "что - строка; где - список; игн_регистр - False; ответ - True"],
        ["сырники", ["кто-то ест Сырники", "кто-то ест грибы"], False, False, "что - строка; где - список; игн_регистр - False; ответ - False"],
        ["сырники", ["кто-то ест Сырники", "кто-то ест грибы"], True, True, "что - строка; где - список; игн_регистр - True; ответ - True"],
        ["пряники", ["кто-то ест Сырники", "кто-то ест грибы"], True, False, "что - строка; где - список; игн_регистр - True; ответ - False"],
        [["Грибы", "Сырники"], "Кто-то ест Грибы", False, True, "что - список; где - строка; игн_регистр - False; ответ - True"],
        [["Грибы", "Сырники"], "Кто-то ест грибы", False, False, "что - список; где - строка; игн_регистр - False; ответ - False"],
        [["Грибы", "Сырники"], "Кто-то ест грибы", True, True, "что - список; где - строка; игн_регистр - True; ответ - True"],
        [["Грибы", "Сырники"], "Кто-то ест пряники", True, False, "что - список; где - строка; игн_регистр - True; ответ - False"],
        [["Грибы", "Сырники"], ["кто-то ест Сырники", "кто-то ест Грибы"], False, True, "что - список; где - список; игн_регистр - False; ответ - True"],
        [["Грибы", "Сырники"], ["кто-то ест сырники", "кто-то ест грибы"], False, False, "что - список; где - список; игн_регистр - False; ответ - False"],
        [["Грибы", "Сырники"], ["кто-то ест сырники", "кто-то ест грибы"], True, True, "что - список; где - список; игн_регистр - True; ответ - True"],
        [["Грибы", "Сырники"], ["кто-то ест пряники", "кто-то ест пельмени"], True, False, "что - список; где - список; игн_регистр - True; ответ - False"],
    ]
    for вариант in варианты:
        что, где, игн_регистр, ответ, название = вариант
        тест(что, где, игн_регистр, ответ, название)
    print("---------------------")
    print(f"Всего текста - {len(пройдено) + len(не_пройдено)}")
    print(f"Из них пройдено - {len(пройдено)}")
    print(f"Не пройдено - {len(не_пройдено)}")
тесты()


from setuptools import setup
setup(
    name="xanter", # имя модуля, которое будет исполшьзоваться при установке через pip unstall
    version="0.3", # версия модуля, при каждом обновлении её надо увеличивать
    description="Мои личные инструменты", # в описании проекта, его можно не использовать
    packages=["xanter"], # список с названиями папок, которые нужно залить
    author_email="ulianaantonova651@gmail.com", # почта автора
    zip_safe=False, # пароль, можно не указывать
    install_requires = [], # список доп. модулей, которые необходимы для использования вашего модуля
)